from si_tests.deployments.utils import (extra_context, file_utils,
                                        wait_utils, commons, kubectl_utils)
from si_tests.deployments.utils.namespace import NAMESPACE


def deploy_metallb(os_manager, timeout):
    context = extra_context.ExtraContext().metallb
    kubectl = kubectl_utils.Kubectl()

    metallb_yaml = file_utils.get_release("3rd-party/30-metallb.yaml")
    metallb_data = file_utils.get_yaml_content(metallb_yaml)
    if context and context.get("metallb_address_pools"):
        commons.LOG.info("Update metallb.yaml")
        config = metallb_data["spec"]["releases"][0]["values"]["configInline"]
        config["address-pools"][0]["addresses"] = context.get(
            "metallb_address_pools")

    new_yaml = file_utils.get_new_name(metallb_yaml)
    commons.LOG.info("Save new metallb yaml to %s", new_yaml)
    file_utils.save_to_yaml(metallb_data, new_yaml)
    commons.LOG.info("Deploy metallb")
    kubectl.apply(new_yaml)

    wait = wait_utils.Waiter(os_manager, timeout, NAMESPACE.metallb)
    commons.LOG.info("Wait till metallb controller starts")
    wait.deployment("metallb-controller")

    commons.LOG.info("Wait till metallb speakers start")
    wait.daemonset("metallb-speaker")

    commons.LOG.info("Metallb successfully deployed")
