import os
import yaml

from si_tests import logger
from si_tests import settings
from si_tests.managers.kaas_manager import Manager
from si_tests.utils import templates, utils, scale_lab_trend_client


LOG = logger.logger


def test_openstack_rally():
    kaas_manager = Manager(kubeconfig=settings.KUBECONFIG_PATH)
    kubectl_client = kaas_manager.api

    ns = settings.RALLY_OPENSTACK_NAMESPACE
    configmap_name = settings.RALLY_OPENSTACK_CONFIGMAP_NAME
    pod_name = settings.RALLY_OPENSTACK_POD_NAME

    image = settings.RALLY_OPENSTACK_IMAGE
    scenarios_repo = settings.RALLY_OPENSTACK_SCENARIOS_REPO
    scenarios_branch = settings.RALLY_OPENSTACK_SCENARIOS_BRANCH
    enable_scenarios = settings.RALLY_OPENSTACK_ENABLE_SCENARIOS
    skip_scenarios = settings.RALLY_OPENSTACK_SKIP_SCENARIOS
    task_args = settings.RALLY_OPENSTACK_RALLY_TASK_ARGS

    env_type = settings.RALLY_ENV_TYPE
    env_name = settings.RALLY_ENV_NAME

    upload_report = settings.RALLY_UPLOAD_SCALE_LAB_TRENDS

    timeout = settings.RALLY_OPENSTACK_WAIT_TIMEOUT

    LOG.info(f"Create ConfigMap {configmap_name}")
    config_map_config = yaml.load(
        templates.render_template(
            settings.RALLY_OPENSTACK_CONFIGMAP_YAML,
            {
                'RALLY_OPENSTACK_CONFIGMAP_NAME': configmap_name
            }
        ),
        Loader=yaml.SafeLoader
    )
    if enable_scenarios:
        config_map_config['data']['enable-scenarios'] = enable_scenarios

    if skip_scenarios:
        config_map_config['data']['skip-scenarios'] = skip_scenarios

    if task_args:
        d = yaml.load(task_args, Loader=yaml.SafeLoader)
        s = yaml.load(config_map_config['data']['task-params.yaml'], Loader=yaml.SafeLoader)
        utils.merge_dicts(s, d)
        config_map_config['data']['task-params.yaml'] = yaml.dump(s)

    if kubectl_client.configmaps.present(name=configmap_name, namespace=ns):
        LOG.warning(f"ConfigMap '{configmap_name}' already exists. Deleting..")
        kubectl_client.api_core.delete_namespaced_config_map(configmap_name, ns)

    kubectl_client.configmaps.create(name=configmap_name, namespace=ns, body=config_map_config)

    LOG.info(f"Create Pod {configmap_name}")
    # FIXME(alexz): must be rewrited to utils\template usage?
    pod_config = yaml.load(
        templates.render_template(
            settings.RALLY_OPENSTACK_POD_YAML,
            {
                'RALLY_OPENSTACK_POD_NAME': pod_name,
                'RALLY_OPENSTACK_IMAGE': image,
                'RALLY_SCENARIOS_REPO': scenarios_repo,
                'RALLY_SCENARIOS_BRANCH': scenarios_branch
            }
        ),
        Loader=yaml.SafeLoader
    )
    if kubectl_client.pods.present(name=pod_name, namespace=ns):
        LOG.warning(f"Pod '{pod_name}' already exists. Deleting..")
        kubectl_client.api_core.delete_namespaced_pod(pod_name, ns)

    pod = kubectl_client.pods.create(name=pod_name, namespace=ns, body=pod_config)

    try:
        # NIT: if container  in not "running", means rally already in progress-since rally-starts in initContainer
        LOG.info(f"Wait pod for Running {pod_name}")
        pod.wait_phase('Running', timeout=timeout, interval=120)
    except Exception:
        LOG.error("Task failed")
        raise
    finally:
        LOG.info("Copy artifacts")
        pod.cp_from_pod(source_dir='/var/rally/results')

        if upload_report:
            sltc = scale_lab_trend_client.ScaleLabTrendClient(
                kubeconfig=settings.KUBECONFIG_PATH,
                kubeconfig_mgmt=settings.KUBECONFIG_MGMT_PATH
            )

            sltc.uploadReport(
                env_type=env_type,
                env_name=env_name,
                path=os.path.join(settings.ARTIFACTS_DIR, 'report.json')
            )

            sltc.uploadCluster(namespace='child-ns')
            sltc.uploadBareMetalHosts(namespace='child-ns')
            sltc.uploadMachines(namespace='child-ns')
            sltc.uploadOpenStackDeployment(namespace='openstack')

    LOG.info("Done")
