import requests
import json

from si_tests.logger import logger
from si_tests import settings
from si_tests.managers.kaas_manager import Manager


class ScaleLabTrendClient:
    _kaas_manager: Manager = None
    _kaas_manager_mgmt: Manager = None

    _report_id: str = ''

    def __init__(self, kubeconfig=None, kubeconfig_mgmt=None, kaas_manager=None, kaas_manager_mgmt=None):
        if kaas_manager:
            self._kaas_manager = kaas_manager
        elif kubeconfig:
            self._kaas_manager = Manager(kubeconfig=kubeconfig)

        if kaas_manager_mgmt:
            self._kaas_manager_mgmt = kaas_manager_mgmt
        elif kubeconfig_mgmt:
            self._kaas_manager_mgmt = Manager(kubeconfig=kubeconfig_mgmt)

    def uploadReport(self, env_type, env_name, path):
        logger.info("ScaleLabTrends upload Rally report: {}".format(path))

        with open(path, 'r') as f:
            resp = requests.post(
                url=settings.RALLY_UPLOAD_SCALE_LAB_TRENDS_URL + "/api/report",
                data={
                    'env_type': env_type,
                    'env_name': env_name,
                },
                files={
                    'report': f
                }
            )
            resp.raise_for_status()

            self._report_id = resp.json()['report_id']

    def uploadBareMetalHosts(self, namespace=None):
        # fields = ['kind', 'metadata', 'spec']

        logger.info("ScaleLabTrends upload BareMetalHosts")

        bmh_list = self._kaas_manager_mgmt.api.kaas_baremetalhosts.list(namespace=namespace)
        for bmh in bmh_list:
            data = {k: bmh.data[k] for k in bmh.model.attribute_map.keys()}

            logger.info("Upload BareMetalHost: {}".format(bmh.name))
            resp = requests.post(
                url=settings.RALLY_UPLOAD_SCALE_LAB_TRENDS_URL + "/api/report/" + self._report_id + "/file",
                files={
                    'bmh': json.dumps(data, default=str)
                }
            )
            resp.raise_for_status()

    def uploadMachines(self, namespace=None):
        # fields = ['kind', 'metadata', 'spec']

        logger.info("ScaleLabTrends upload Machines")

        machine_list = self._kaas_manager_mgmt.api.kaas_machines.list(namespace=namespace)
        for machine in machine_list:
            data = {k: machine.data[k] for k in machine.model.attribute_map.keys()}

            logger.info("Upload Machine: {}".format(machine.name))
            resp = requests.post(
                url=settings.RALLY_UPLOAD_SCALE_LAB_TRENDS_URL + "/api/report/" + self._report_id + "/file",
                files={
                    'machine': json.dumps(data, default=str)
                }
            )
            resp.raise_for_status()

    def uploadCluster(self, namespace=None):
        # fields = ['kind', 'metadata', 'spec']

        logger.info("ScaleLabTrends upload Cluster")

        cluster_list = self._kaas_manager_mgmt.api.kaas_clusters.list(namespace=namespace)
        for cluster in cluster_list:
            data = {k: cluster.data[k] for k in cluster.model.attribute_map.keys()}

            logger.info("Upload Cluster: {}".format(cluster.name))
            resp = requests.post(
                url=settings.RALLY_UPLOAD_SCALE_LAB_TRENDS_URL + "/api/report/" + self._report_id + "/file",
                files={
                    'cluster': json.dumps(data, default=str)
                }
            )
            resp.raise_for_status()

    def uploadOpenStackDeployment(self, namespace=None):
        # fields = ['kind', 'metadata', 'spec']

        logger.info("ScaleLabTrends upload OpenStackDeployment")

        osdpl_list = self._kaas_manager.api.openstackdeployment.list(namespace=namespace)
        for osdpl in osdpl_list:
            data = {k: osdpl.data[k] for k in osdpl.model.attribute_map.keys()}

            logger.info("Upload OpenStackDeployment: {}".format(osdpl.name))
            resp = requests.post(
                url=settings.RALLY_UPLOAD_SCALE_LAB_TRENDS_URL + "/api/report/" + self._report_id + "/file",
                files={
                    'cluster': json.dumps(data, default=str)
                }
            )
            resp.raise_for_status()
